/* The Virtual Method Table (vmt) for Base */
struct Base_vmt {
 int (*GetX)(struct Base*);
 int (*GetY)(struct Base*);
 void (*Move)(struct Base*);
};

/* The class layout for Base */
struct Base {
 struct Base_vmt *vmt;
 int x;
 int y;
};

/* Base::GetX() */
int Base_GetX(struct Base *this) {
 return this->x;
}

/* Base::GetY() */
int Base_GetY(struct Base *this) {
 return this->y;
}

/*
 This is not used but is what Base's VMT would look like, with one
 abstract (missing, null) slot for Move()
 struct Base_vmt base_vmt = { Base_GetX, Base_GetY, 0 };
*/

/* The Virtual Method Table (vmt) for C1 */
struct C1_vmt {
 int (*GetX)(struct C1*);
 int (*GetY)(struct C1*);
 void (*Move)(struct C1*);
};

/* The class layout for C1 */
struct C1 {
 struct C1_vmt *vmt;
 int x;
 int y;
};

/* C1::Move() */
void C1_Move(struct C1 *this) {
 return;
}

struct C1_vmt c1_vmt = { Base_GetX, Base_GetY, C1_Move };

/* The Virtual Method Table (vmt) for C2 */
struct C2_vmt {
 int (*GetX)(struct C2*);
 int (*GetY)(struct C2*);
 void (*Move)(struct C2*);
};

/* The class layout for C2 */
struct C2 {
 struct C2_vmt *vmt;
 int x;
 int y;
};

/* C2::Move() */
void C2_Move(struct C2 *this) {
 this->x++;
}

struct C2_vmt c2_vmt = { Base_GetX, Base_GetY, C2_Move };

/* The Virtual Method Table (vmt) for C3 */
struct C3_vmt {
 int (*GetX)(struct C3*);
 int (*GetY)(struct C3*);
 void (*Move)(struct C3*);
};

/* The class layout for C3 */
struct C3 {
 struct C3_vmt *vmt;
 int x;
 int y;
};

/* C3::Move() */
void C3_Move(struct C3 *this) {
 this->x++;
 this->y++;
}

struct C3_vmt c3_vmt = { Base_GetX, Base_GetY, C3_Move };

/* The Virtual Method Table (vmt) for C4 */
struct C4_vmt {
 int (*GetX)(struct C4*);
 int (*GetY)(struct C4*);
 void (*Move)(struct C4*);
 int (*GetZ)(struct C4*);
};

/* The class layout for C4 */
struct C4 {
 struct C4_vmt *vmt;
 int x;
 int y;
 int z;
};

/* C4::Move() */
void C4_Move(struct C4 *this) {
 this->x++;
 this->y++;
 this->z++;
}

/* C4::GetZ() */
int C4_GetZ(struct C4 *this) {
 return this->z;
}

struct C4_vmt c4_vmt = { Base_GetX, Base_GetY, C4_Move, C4_GetZ };

/* This is how the objects would look after initialisation by their
 constructors (not shown here), for simplicity. This is not valid C. */
struct C4 *c4 = { &c4_vmt, 2, 5, 7 };
struct C3 *c3 = { &c3_vmt, 1, 2 };
struct C2 *c2 = { &c2_vmt, 5, 5 };
struct C1 *c1 = { &c1_vmt, 8, 3 };

int v1;
int v2;
int v3;

void dostuff() {
 c4->vmt->Move(c4);
 v1 = c4->vmt->GetZ(c4);
 c2->vmt->Move(c2);
 v2 = c2->vmt->GetX(c2);
 c3->vmt->Move(c3);
 v3 = c3->vmt->GetY(c3);
 c1->vmt->Move(c1);
 v2 += c1->vmt->GetX(c1);
}

/* called by stub */
dostuff();
